from typing import List

from dataHubPy.connector.connector import Connector
from dataHubPy.connector.mysql import MysqlConnector
from dataHubPy.connector.postgresql import PostgresqlConnector
from dataHubPy.connector.s3_file import S3Connector
from dataHubPy.connector.http_file import HttpConnector


class ConnectorFactory:
    def __init__(self, configuration_map: dict) -> None:
        super().__init__()
        self._configuration_map = configuration_map

    @staticmethod
    def wrap(info: dict):
        connector_type = info["type"]
        if "mysql" == connector_type:
            return MysqlConnector().init(info)
        elif "postgresql" == connector_type:
            return PostgresqlConnector().init(info)
        elif "s3" == connector_type:
            return S3Connector().init(info)
        elif "http" == connector_type:
            return HttpConnector().init(info)
        raise Exception("type({}) not support".format(connector_type))

    def input_connector_list(self) -> List[Connector]:
        if "input" not in self._configuration_map:
            return []
        input_connector_map_list = self._configuration_map["input"]
        return [ConnectorFactory.wrap(input_connector_map) for input_connector_map in input_connector_map_list]

    def output_connector_list(self) -> List[Connector]:
        if "output" not in self._configuration_map:
            return []
        output_connector_map_list = self._configuration_map["output"]
        return [ConnectorFactory.wrap(output_connector_map) for output_connector_map in output_connector_map_list]



import pandas


class Connector:

    def __init__(self):
        super().__init__()

    def init(self, info: dict):
        self.__dict__.update(info)
        return self

    def read_as_data_frame(self, table_name: str = "") -> pandas.DataFrame:
        pass

    def write_data_frame(self, df: pandas.DataFrame, table_name: str = "") -> None:
        pass

    def cached_file(self):
        raise Exception("Not Implement yet.")



import pandas
import requests
from urllib import request
from dataHubPy.connector.connector import Connector


class HttpConnector(Connector):

    def __init__(
            self,
            url: str = "",
            request_type: str = "get",
            headers: dict = None,
            param: dict = None,
            policy: str = "replace",
            file_type: str = "csv"
    ) -> None:
        super().__init__()
        self.url = url
        self.request_type = request_type
        self.headers = headers
        self.param = param
        self.persist_policy = policy
        self.persist_file_type = file_type

    def get_url(self):
        return self.url

    def get_request_type(self):
        return self.request_type

    def get_param(self):
        return self.param

    def get_headers(self):
        return self.headers

    def get_persist_file_type(self):
        return self.persist_file_type

    def get_persist_policy(self):
        return self.persist_policy

    def init(self, info: dict):
        self.url = info.get("url", "")
        self.request_type = info.get("request", {}).get("type", "get")
        self.headers = info.get("request", {}).get("headers", {})
        self.param = info.get("request", {}).get("param", {})
        self.persist_policy = info.get("persist", {}).get("policy", "")
        return self

    def __eq__(self, other):
        if not isinstance(other, HttpConnector):
            return False
        return (self.get_url() == other.get_url()
                and self.get_request_type() == other.get_request_type()
                and self.get_param() == other.get_param()
                and self.get_headers() == other.get_headers()
                and self.get_persist_file_type() == other.get_persist_file_type()
                and self.get_persist_policy() == other.get_persist_policy()
                )

    def read_as_data_frame(self, table_name: str = "") -> pandas.DataFrame:
        if not table_name or table_name == "":
            raise Exception("please specify a path that could be used to save http response.")
        if self.get_request_type().lower() == 'get':
            if self.get_param() is None:
                request.urlretrieve(self.get_url(), table_name)
            else:
                response = requests.get(url=self.get_url(),
                                        params=self.get_param(),
                                        headers=self.get_headers())
                print(response)
        else:
            response = requests.post(url=self.get_url(),
                                     data=self.get_param(),
                                     headers=self.get_headers())
            with open(table_name, "wb") as f:
                f.write(response.content)

        if self.get_persist_file_type().lower() == "csv":
            return pandas.read_csv(table_name)
        elif self.get_persist_file_type().lower() == "xlsx":
            return pandas.read_excel(table_name)
        elif self.get_persist_file_type().lower() == "json":
            return pandas.read_json(table_name, typ='series')
        else:
            raise Exception("unacceptable file type.")

    def write_data_frame(self, df: pandas.DataFrame, table_name: str = "") -> None:
        pass



import setuptools


with open("README.md", "r") as fh:
    long_description = fh.read()


setuptools.setup(
    name="datahub-algo-base",
    version="0.0.17",
    author="Aaron",
    author_email="byang628@zhejianglab.com",
    description="A template for algorithm node",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitea-ops.lab.zjvis.net/flint/data-hub",
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.7",
    install_requires=[
        "waitress",
        'minio',
        'numpy',
        'pandas',
        'Flask',
        'requests',
        'psycopg2',
        'PyMySQL',
        'PyYAML',
        'scikit-learn',
        'SQLAlchemy',
        'impyute'
    ]

)



from typing import List

from dataHubPy.algorithm.algorithm import Algorithm
from dataHubPy.connector.connector import Connector


class Copy(Algorithm):
    def __init__(self, configuration: dict) -> None:
        super().__init__(configuration)

    def __init_subclass__(cls, **kwargs):
        super().__init_subclass__(**kwargs)

    def run(
            self,
            connector_in_list: List[Connector],
            connector_out_list: List[Connector],
            cache_list: List[Connector]
    ) -> dict:
        connector_in = connector_in_list[0]
        connector_out = connector_out_list[0]
        data = connector_in.read_as_data_frame(self._input_data_item)
        connector_out.write_data_frame(data, self._output_data_item)
        return {"success": True}
